@@ -301,8 +301,7 @@ calendar_selector_constructed (GObject *object)
 	g_object_unref (source);
 
 	/* Chain up to parent's constructed() method. */
-	G_OBJECT_CLASS (e_calendar_selector_parent_class)->
-		constructed (object);
+	G_OBJECT_CLASS (e_calendar_selector_parent_class)->constructed (object);
 }
 
 static gboolean
