@@ -492,12 +492,11 @@ e_composer_paste_html (EMsgComposer *composer,
 	editor = e_msg_composer_get_editor (composer);
 	view = e_html_editor_get_view (editor);
 	editor_selection = e_html_editor_view_get_selection (view);
+	/* If Web View doesn't have focus, focus it */
+	if (!gtk_widget_has_focus (GTK_WIDGET (view)))
+		gtk_widget_grab_focus (GTK_WIDGET (view));
 	e_html_editor_selection_insert_html (editor_selection, html);
 
-	e_html_editor_view_check_magic_links (view, FALSE);
-	e_html_editor_view_force_spell_check (view);
-
-	e_html_editor_selection_scroll_to_caret (editor_selection);
 	g_free (html);
 
 	return TRUE;
@@ -607,10 +606,6 @@ e_composer_paste_text (EMsgComposer *composer,
 
 	e_html_editor_selection_insert_text (editor_selection, text);
 
-	e_html_editor_view_check_magic_links (view, FALSE);
-	e_html_editor_view_force_spell_check (view);
-	e_html_editor_selection_scroll_to_caret (editor_selection);
-
 	g_free (text);
 
 	return TRUE;
@@ -1177,6 +1172,7 @@ insert:
 
 			body = webkit_dom_document_get_body (document);
 			element = webkit_dom_document_create_element (document, "DIV", NULL);
+			webkit_dom_element_set_class_name (element, "-x-evo-signature-wrapper");
 
 			webkit_dom_html_element_set_inner_html (
 				WEBKIT_DOM_HTML_ELEMENT (element), html_buffer->str, NULL);
