@@ -655,6 +655,21 @@ client_cache_cal_connect_cb (GObject *source_object,
 	client_data_unref (client_data);
 }
 
+static void
+client_cache_source_allow_auth_prompt_done_cb (GObject *source_object,
+					       GAsyncResult *result,
+					       gpointer user_data)
+{
+	GError *local_error = NULL;
+
+	e_source_allow_auth_prompt_finish (E_SOURCE (source_object), result, &local_error);
+
+	if (local_error) {
+		g_debug ("%s: Failed with: %s", G_STRFUNC, local_error->message);
+		g_clear_error (&local_error);
+	}
+}
+
 static void
 client_cache_source_removed_cb (ESourceRegistry *registry,
                                 ESource *source,
@@ -680,6 +695,11 @@ client_cache_source_disabled_cb (ESourceRegistry *registry,
 	client_cache = g_weak_ref_get (weak_ref);
 
 	if (client_cache != NULL) {
+		/* There is not much interest in the result, it just
+		   makes sure a password prompt will be shown the next
+		   time it is needed. */
+		e_source_allow_auth_prompt (source, NULL, client_cache_source_allow_auth_prompt_done_cb, NULL);
+
 		client_ht_remove (client_cache, source);
 		g_object_unref (client_cache);
 	}
@@ -1047,28 +1067,6 @@ e_client_cache_ref_registry (EClientCache *client_cache)
 	return g_object_ref (client_cache->priv->registry);
 }
 
-typedef struct _GetClientSyncData {
-	GMutex mutex;
-	EAsyncClosure *closure;
-} GetClientSyncData;
-
-static void
-client_cache_get_client_sync_cb (GObject *source_object,
-				 GAsyncResult *result,
-				 gpointer user_data)
-{
-	GetClientSyncData *data = user_data;
-
-	g_return_if_fail (E_IS_CLIENT_CACHE (source_object));
-	g_return_if_fail (data != NULL);
-
-	g_mutex_lock (&data->mutex);
-
-	e_async_closure_callback (source_object, result, data->closure);
-
-	g_mutex_unlock (&data->mutex);
-}
-
 /**
  * e_client_cache_get_client_sync:
  * @client_cache: an #EClientCache
@@ -1116,7 +1114,7 @@ e_client_cache_get_client_sync (EClientCache *client_cache,
                                 GCancellable *cancellable,
                                 GError **error)
 {
-	GetClientSyncData data;
+	EAsyncClosure *closure;
 	GAsyncResult *result;
 	EClient *client;
 
@@ -1124,29 +1122,18 @@ e_client_cache_get_client_sync (EClientCache *client_cache,
 	g_return_val_if_fail (E_IS_SOURCE (source), NULL);
 	g_return_val_if_fail (extension_name != NULL, NULL);
 
-	g_mutex_init (&data.mutex);
-	g_mutex_lock (&data.mutex);
+	closure = e_async_closure_new ();
 
 	e_client_cache_get_client (
 		client_cache, source, extension_name,cancellable,
-		client_cache_get_client_sync_cb, &data);
+		e_async_closure_callback, closure);
 
-	/* This is needed, because e_async_closure_new() pushes its own thread default main context,
-	   which was later taken into an EClient within e_client_cache_get_client(), but that's wrong,
-	   because that main context effectively dies at the end of this function. */
-	data.closure = e_async_closure_new ();
-
-	g_mutex_unlock (&data.mutex);
-
-	result = e_async_closure_wait (data.closure);
+	result = e_async_closure_wait (closure);
 
 	client = e_client_cache_get_client_finish (
 		client_cache, result, error);
 
-	g_mutex_lock (&data.mutex);
-	e_async_closure_free (data.closure);
-	g_mutex_unlock (&data.mutex);
-	g_mutex_clear (&data.mutex);
+	e_async_closure_free (closure);
 
 	return client;
 }
@@ -1289,8 +1276,6 @@ e_client_cache_get_client (EClientCache *client_cache,
 	g_warn_if_reached ();  /* Should never happen. */
 
 exit:
-	if (client_data)
-		client_data_unref (client_data);
 	g_object_unref (simple);
 }
 
