@@ -598,11 +598,13 @@ html_editor_constructed (GObject *object)
 	EHTMLEditor *editor = E_HTML_EDITOR (object);
 	EHTMLEditorPrivate *priv = editor->priv;
 	GtkIMMulticontext *im_context;
-
 	GtkWidget *widget;
 	GtkToolbar *toolbar;
 	GtkToolItem *tool_item;
 
+	/* Chain up to parent's method. */
+	G_OBJECT_CLASS (e_html_editor_parent_class)->constructed (object);
+
 	/* Construct the editing toolbars. */
 
 	widget = e_html_editor_get_managed_widget (editor, "/edit-toolbar");
