@@ -606,20 +606,20 @@ style_updated_cb (EWebView *web_view)
 	gchar *style;
 	GtkStateFlags state_flags;
 	GtkStyleContext *style_context;
-	GtkWidgetPath *widget_path;
+	gboolean backdrop;
 
 	state_flags = gtk_widget_get_state_flags (GTK_WIDGET (web_view));
-	style_context = gtk_style_context_new ();
-	widget_path = gtk_widget_path_new ();
-	gtk_widget_path_append_type (widget_path, GTK_TYPE_WINDOW);
-	gtk_style_context_set_path (style_context, widget_path);
-	gtk_style_context_add_class (style_context, GTK_STYLE_CLASS_ENTRY);
+	style_context = gtk_widget_get_style_context (GTK_WIDGET (web_view));
+	backdrop = (state_flags & GTK_STATE_FLAG_BACKDROP) != 0;
+
+	if (gtk_style_context_lookup_color (
+			style_context,
+			backdrop ? "theme_unfocused_base_color" : "theme_base_color",
+			&color))
+		color_value = g_strdup_printf ("#%06x", e_rgba_to_value (&color));
+	else
+		color_value = g_strdup (E_UTILS_DEFAULT_THEME_BASE_COLOR);
 
-	gtk_style_context_get_background_color (
-		style_context,
-		state_flags | GTK_STATE_FLAG_FOCUSED,
-		&color);
-	color_value = g_strdup_printf ("#%06x", e_rgba_to_value (&color));
 	style = g_strconcat ("background-color: ", color_value, ";", NULL);
 
 	e_web_view_add_css_rule_into_style_sheet (
@@ -631,11 +631,14 @@ style_updated_cb (EWebView *web_view)
 	g_free (color_value);
 	g_free (style);
 
-	gtk_style_context_get_color (
-		style_context,
-		state_flags | GTK_STATE_FLAG_FOCUSED,
-		&color);
-	color_value = g_strdup_printf ("#%06x", e_rgba_to_value (&color));
+	if (gtk_style_context_lookup_color (
+			style_context,
+			backdrop ? "theme_unfocused_fg_color" : "theme_fg_color",
+			&color))
+		color_value = g_strdup_printf ("#%06x", e_rgba_to_value (&color));
+	else
+		color_value = g_strdup (E_UTILS_DEFAULT_THEME_FG_COLOR);
+
 	style = g_strconcat ("color: ", color_value, ";", NULL);
 
 	e_web_view_add_css_rule_into_style_sheet (
@@ -644,9 +647,6 @@ style_updated_cb (EWebView *web_view)
 		".-e-web-view-text-color",
 		style);
 
-	gtk_widget_path_free (widget_path);
-	g_object_unref (style_context);
-
 	g_free (color_value);
 	g_free (style);
 }
@@ -2620,6 +2620,27 @@ e_web_view_update_actions (EWebView *web_view)
 	g_signal_emit (web_view, signals[UPDATE_ACTIONS], 0);
 }
 
+static gboolean
+element_is_in_pre_tag (WebKitDOMNode *node)
+{
+	WebKitDOMElement *element;
+
+	if (!node)
+		return FALSE;
+
+	while (element = webkit_dom_node_get_parent_element (node), element) {
+		node = WEBKIT_DOM_NODE (element);
+
+		if (WEBKIT_DOM_IS_HTML_PRE_ELEMENT (element)) {
+			return TRUE;
+		} else if (WEBKIT_DOM_IS_HTML_IFRAME_ELEMENT (element)) {
+			break;
+		}
+	}
+
+	return FALSE;
+}
+
 static gchar *
 web_view_get_frame_selection_html (WebKitDOMElement *iframe)
 {
@@ -2640,6 +2661,9 @@ web_view_get_frame_selection_html (WebKitDOMElement *iframe)
 
 		range = webkit_dom_dom_selection_get_range_at (selection, 0, NULL);
 		if (range != NULL) {
+			gchar *inner_html;
+			WebKitDOMNode *node;
+
 			fragment = webkit_dom_range_clone_contents (
 				range, NULL);
 
@@ -2649,8 +2673,16 @@ web_view_get_frame_selection_html (WebKitDOMElement *iframe)
 				WEBKIT_DOM_NODE (element),
 				WEBKIT_DOM_NODE (fragment), NULL);
 
-			return webkit_dom_html_element_get_inner_html (
+			inner_html = webkit_dom_html_element_get_inner_html (
 				WEBKIT_DOM_HTML_ELEMENT (element));
+			node = webkit_dom_range_get_start_container (range, NULL);
+			if (element_is_in_pre_tag (node)) {
+				gchar *tmp = inner_html;
+				inner_html = g_strconcat ("<pre>", tmp, "</pre>", NULL);
+				g_free (tmp);
+			}
+
+			return inner_html;
 		}
 	}
 
