@@ -1048,7 +1048,7 @@ em_utils_composer_print_cb (EMsgComposer *composer,
 	/* as long as EMsgComposer uses GtkHTML, use its routine for printing;
 	 * this conditional compile is here rather to not forget to fix this
 	 * once the WebKit-based composer will land */
-#if defined(GTK_TYPE_HTML)
+#if 0 /* defined(GTK_TYPE_HTML) */
 	EWebViewGtkHTML *gtkhtml_web_view;
 	GtkPrintOperation *operation;
 	GError *error = NULL;
