@@ -27,6 +27,9 @@
 #include <glib/gi18n.h>
 
 #include <shell/e-shell.h>
+#include <shell/e-shell-content.h>
+#include <shell/e-shell-view.h>
+#include <shell/e-shell-window.h>
 #include <e-util/e-util.h>
 
 #include "e-mail-account-store.h"
@@ -428,6 +431,108 @@ format_service_name (CamelService *service)
 	return pretty_url;
 }
 
+struct ReportErrorToUIData
+{
+	gchar *display_name;
+	gchar *error_ident;
+	GError *error;
+};
+
+static gboolean
+report_error_to_ui_cb (gpointer user_data)
+{
+	struct ReportErrorToUIData *data = user_data;
+	EShellView *shell_view = NULL;
+
+	g_return_val_if_fail (data != NULL, FALSE);
+	g_return_val_if_fail (data->display_name != NULL, FALSE);
+	g_return_val_if_fail (data->error_ident != NULL, FALSE);
+	g_return_val_if_fail (data->error != NULL, FALSE);
+
+	if (send_recv_dialog) {
+		GtkWidget *parent;
+
+		parent = gtk_widget_get_parent (send_recv_dialog);
+		if (parent && E_IS_SHELL_WINDOW (parent)) {
+			EShellWindow *shell_window = E_SHELL_WINDOW (parent);
+
+			shell_view = e_shell_window_get_shell_view (shell_window, "mail");
+		}
+	}
+
+	if (!shell_view) {
+		EShell *shell;
+		GtkWindow *active_window;
+
+		shell = e_shell_get_default ();
+		active_window = e_shell_get_active_window (shell);
+
+		if (E_IS_SHELL_WINDOW (active_window)) {
+			EShellWindow *shell_window = E_SHELL_WINDOW (active_window);
+
+			shell_view = e_shell_window_get_shell_view (shell_window, "mail");
+		}
+	}
+
+	if (shell_view) {
+		EShellContent *shell_content;
+		EAlertSink *alert_sink;
+		EAlert *alert;
+
+		shell_content = e_shell_view_get_shell_content (shell_view);
+		alert_sink = E_ALERT_SINK (shell_content);
+
+		alert = e_alert_new (data->error_ident, data->display_name, data->error->message, NULL);
+
+		e_alert_sink_submit_alert (alert_sink, alert);
+
+		g_object_unref (alert);
+	} else {
+		/* This may not happen, but just in case... */
+		g_warning ("%s: %s '%s': %s\n", G_STRFUNC, data->error_ident, data->display_name, data->error->message);
+	}
+
+	g_free (data->display_name);
+	g_free (data->error_ident);
+	g_error_free (data->error);
+	g_free (data);
+
+	return FALSE;
+}
+
+static void
+report_error_to_ui (CamelService *service,
+		    const gchar *folder_name,
+		    const GError *error)
+{
+	gchar *tmp = NULL;
+	const gchar *display_name, *ident;
+	struct ReportErrorToUIData *data;
+
+	g_return_if_fail (CAMEL_IS_SERVICE (service));
+	g_return_if_fail (error != NULL);
+
+	if (folder_name) {
+		tmp = g_strdup_printf ("%s: %s",
+			camel_service_get_display_name (service),
+			folder_name);
+		display_name = tmp;
+		ident = "mail:no-refresh-folder";
+	} else {
+		display_name = camel_service_get_display_name (service);
+		ident = "mail:failed-connect";
+	}
+
+	data = g_new0 (struct ReportErrorToUIData, 1);
+	data->display_name = g_strdup (display_name);
+	data->error_ident = g_strdup (ident);
+	data->error = g_error_copy (error);
+
+	g_idle_add_full (G_PRIORITY_DEFAULT, report_error_to_ui_cb, data, NULL);
+
+	g_free (tmp);
+}
+
 static send_info_t
 get_receive_type (CamelService *service)
 {
@@ -531,8 +636,6 @@ build_dialog (GtkWindow *parent,
 		"/org/gnome/evolution/mail/send-recv-window/",
 		E_RESTORE_WINDOW_SIZE);
 
-	gtk_widget_ensure_style ((GtkWidget *) gd);
-
 	container = gtk_dialog_get_action_area (gd);
 	gtk_container_set_border_width (GTK_CONTAINER (container), 6);
 
@@ -1161,21 +1264,18 @@ refresh_folders_exec (struct _refresh_folders_msg *m,
 
 		if (local_error != NULL) {
 			if (!g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-				const gchar *account_name = NULL, *full_name;
+				CamelStore *store = m->store;
+				const gchar *full_name;
 
 				if (folder) {
-					CamelStore *store = camel_folder_get_parent_store (folder);
-
-					account_name = camel_service_get_display_name (CAMEL_SERVICE (store));
+					store = camel_folder_get_parent_store (folder);
 					full_name = camel_folder_get_full_name (folder);
-				} else
+				} else {
+					store = m->store;
 					full_name = (const gchar *) m->folders->pdata[i];
+				}
 
-				g_warning (
-					"Failed to refresh folder '%s%s%s': %s",
-					account_name ? account_name : "",
-					account_name ? ": " : "",
-					full_name, local_error->message);
+				report_error_to_ui (CAMEL_SERVICE (store), full_name, local_error);
 			}
 
 			g_clear_error (&local_error);
@@ -1242,15 +1342,19 @@ receive_update_got_folderinfo (GObject *source_object,
 
 	/* Ignore cancellations. */
 	if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		g_warn_if_fail (info != NULL);
+		g_warn_if_fail (info == NULL);
 		g_error_free (local_error);
 
+		receive_done (send_info);
+
 	/* XXX Need to hand this off to an EAlertSink. */
 	} else if (local_error != NULL) {
-		g_warn_if_fail (info != NULL);
-		g_warning ("%s: %s", G_STRFUNC, local_error->message);
+		g_warn_if_fail (info == NULL);
+		report_error_to_ui (send_info->service, NULL, local_error);
 		g_error_free (local_error);
 
+		receive_done (send_info);
+
 	/* CamelFolderInfo may be NULL even if no error occurred. */
 	} else if (info != NULL) {
 		GPtrArray *folders = g_ptr_array_new ();
