@@ -23,6 +23,7 @@
 #endif
 
 #include "e-cal-shell-view-private.h"
+#include "e-cal-shell-view.h"
 
 /* This is for radio action groups whose value is persistent.  We
  * initialize it to a bogus value to ensure a "changed" signal is
@@ -374,7 +375,6 @@ action_calendar_refresh_cb (GtkAction *action,
 	ESourceSelector *selector;
 	EClient *client = NULL;
 	ESource *source;
-	GError *error = NULL;
 
 	cal_shell_sidebar = cal_shell_view->priv->cal_shell_sidebar;
 	selector = e_cal_shell_sidebar_get_selector (cal_shell_sidebar);
@@ -392,15 +392,7 @@ action_calendar_refresh_cb (GtkAction *action,
 
 	g_return_if_fail (e_client_check_refresh_supported (client));
 
-	e_client_refresh_sync (client, NULL, &error);
-
-	if (error != NULL) {
-		g_warning (
-			"%s: Failed to refresh '%s', %s",
-			G_STRFUNC, e_source_get_display_name (source),
-			error->message);
-		g_error_free (error);
-	}
+	e_cal_shell_view_allow_auth_prompt_and_refresh (E_SHELL_VIEW (cal_shell_view), client);
 
 	g_object_unref (client);
 }
