@@ -22,6 +22,8 @@
 #include <config.h>
 #endif
 
+#include "e-memo-shell-sidebar.h"
+
 #include <string.h>
 #include <glib/gi18n.h>
 
@@ -29,9 +31,6 @@
 #include "calendar/gui/e-memo-list-selector.h"
 #include "calendar/gui/misc.h"
 
-#include "e-memo-shell-content.h"
-#include "e-memo-shell-sidebar.h"
-
 #define E_MEMO_SHELL_SIDEBAR_GET_PRIVATE(obj) \
 	(G_TYPE_INSTANCE_GET_PRIVATE \
 	((obj), E_TYPE_MEMO_SHELL_SIDEBAR, EMemoShellSidebarPrivate))
@@ -350,7 +349,7 @@ memo_shell_sidebar_set_default (EMemoShellSidebar *memo_shell_sidebar,
 	priv->connecting_default_client = g_object_ref (closure->activity);
 
 	e_client_selector_get_client (
-		E_CLIENT_SELECTOR (selector), source,
+		E_CLIENT_SELECTOR (selector), source, TRUE,
 		e_activity_get_cancellable (closure->activity),
 		memo_shell_sidebar_default_connect_cb, closure);
 }
@@ -781,7 +780,7 @@ e_memo_shell_sidebar_add_source (EMemoShellSidebar *memo_shell_sidebar,
 	closure = connect_closure_new (memo_shell_sidebar, source);
 
 	e_client_selector_get_client (
-		E_CLIENT_SELECTOR (selector), source,
+		E_CLIENT_SELECTOR (selector), source, TRUE,
 		e_activity_get_cancellable (closure->activity),
 		memo_shell_sidebar_client_connect_cb, closure);
 }
@@ -798,29 +797,8 @@ e_memo_shell_sidebar_remove_source (EMemoShellSidebar *memo_shell_sidebar,
 
 	selector = e_memo_shell_sidebar_get_selector (memo_shell_sidebar);
 
-	client = e_client_selector_ref_cached_client (E_CLIENT_SELECTOR (selector), source);
-	if (!client) {
-		EShellView *shell_view;
-		EShellContent *shell_content;
-		ECalModel *model;
-		GList *clients, *link;
-
-		shell_view = e_shell_sidebar_get_shell_view (E_SHELL_SIDEBAR (memo_shell_sidebar));
-		shell_content = e_shell_view_get_shell_content (shell_view);
-		model = e_memo_shell_content_get_memo_model (E_MEMO_SHELL_CONTENT (shell_content));
-
-		clients = e_cal_model_list_clients (model);
-		for (link = clients; link; link = g_list_next (link)) {
-			EClient *adept = link->data;
-
-			if (adept && g_strcmp0 (e_source_get_uid (source), e_source_get_uid (e_client_get_source (adept))) == 0) {
-				client = g_object_ref (adept);
-				break;
-			}
-		}
-
-		g_list_free_full (clients, g_object_unref);
-	}
+	client = e_client_selector_ref_cached_client (
+		E_CLIENT_SELECTOR (selector), source);
 
 	if (client != NULL) {
 		memo_shell_sidebar_emit_client_removed (
