@@ -28,7 +28,6 @@
 
 #include "em-mailer-prefs.h"
 
-#include <gtkhtml/gtkhtml-properties.h>
 #include <libxml/tree.h>
 
 #include <shell/e-shell-utils.h>
@@ -78,9 +77,9 @@ static const struct {
 	gint days;
 } empty_trash_frequency[] = {
 	{ N_("On exit, every time"), 0 },
-	{ N_("On exit, once per day"), 1 },
-	{ N_("On exit, once per week"), 7 },
-	{ N_("On exit, once per month"), 30 },
+	{ N_("Once per day"), 1 },
+	{ N_("Once per week"), 7 },
+	{ N_("Once per month"), 30 },
 	{ N_("Immediately, on folder leave"), -1 }
 };
 
@@ -251,6 +250,8 @@ jh_add_cb (GtkWidget *widget,
 	gchar *tok;
 	const gchar *name, *value;
 
+	g_type_ensure (E_TYPE_MAIL_JUNK_OPTIONS);
+
 	e_load_ui_builder_definition (builder, "mail-config.ui");
 	dialog = e_builder_get_widget (builder, "add-custom-junk-header");
 	jh_dialog_entry_changed_cb (NULL, builder);
@@ -736,7 +737,7 @@ image_loading_policy_always_cb (GtkToggleButton *toggle_button)
 
 		g_settings_set_enum (
 			settings, "image-loading-policy",
-			E_MAIL_IMAGE_LOADING_POLICY_ALWAYS);
+			E_IMAGE_LOADING_POLICY_ALWAYS);
 
 		g_object_unref (settings);
 	}
@@ -752,7 +753,7 @@ image_loading_policy_sometimes_cb (GtkToggleButton *toggle_button)
 
 		g_settings_set_enum (
 			settings, "image-loading-policy",
-			E_MAIL_IMAGE_LOADING_POLICY_SOMETIMES);
+			E_IMAGE_LOADING_POLICY_SOMETIMES);
 
 		g_object_unref (settings);
 	}
@@ -768,7 +769,7 @@ image_loading_policy_never_cb (GtkToggleButton *toggle_button)
 
 		g_settings_set_enum (
 			settings, "image-loading-policy",
-			E_MAIL_IMAGE_LOADING_POLICY_NEVER);
+			E_IMAGE_LOADING_POLICY_NEVER);
 
 		g_object_unref (settings);
 	}
@@ -842,8 +843,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 
 	/* Make sure our custom widget classes are registered with
 	 * GType before we load the GtkBuilder definition file. */
-	E_TYPE_MAIL_JUNK_OPTIONS;
-	EM_TYPE_FOLDER_SELECTION_BUTTON;
+	g_type_ensure (E_TYPE_MAIL_JUNK_OPTIONS);
 
 	prefs->builder = gtk_builder_new ();
 	e_load_ui_builder_definition (prefs->builder, "mail-config.ui");
@@ -1005,7 +1005,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 		prefs->builder, "radImagesNever");
 	gtk_toggle_button_set_active (
 		GTK_TOGGLE_BUTTON (widget),
-		val == E_MAIL_IMAGE_LOADING_POLICY_NEVER);
+		val == E_IMAGE_LOADING_POLICY_NEVER);
 	gtk_widget_set_sensitive (widget, writable);
 
 	g_signal_connect (
@@ -1016,7 +1016,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 		prefs->builder, "radImagesSometimes");
 	gtk_toggle_button_set_active (
 		GTK_TOGGLE_BUTTON (widget),
-		val == E_MAIL_IMAGE_LOADING_POLICY_SOMETIMES);
+		val == E_IMAGE_LOADING_POLICY_SOMETIMES);
 	gtk_widget_set_sensitive (widget, writable);
 
 	g_signal_connect (
@@ -1027,7 +1027,7 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 		prefs->builder, "radImagesAlways");
 	gtk_toggle_button_set_active (
 		GTK_TOGGLE_BUTTON (widget),
-		val == E_MAIL_IMAGE_LOADING_POLICY_ALWAYS);
+		val == E_IMAGE_LOADING_POLICY_ALWAYS);
 	gtk_widget_set_sensitive (widget, writable);
 
 	g_signal_connect (
@@ -1065,6 +1065,16 @@ em_mailer_prefs_construct (EMMailerPrefs *prefs,
 		widget, "active",
 		G_SETTINGS_BIND_DEFAULT);
 
+	widget = e_builder_get_widget (prefs->builder, "search_gravatar");
+	g_settings_bind (
+		settings, "search-gravatar-for-photo",
+		widget, "active",
+		G_SETTINGS_BIND_DEFAULT);
+	g_settings_bind (
+		settings, "show-sender-photo",
+		widget, "sensitive",
+		G_SETTINGS_BIND_GET);
+
 	/* always de-sensitised until the user types something in the entry */
 	prefs->add_header = GTK_BUTTON (e_builder_get_widget (prefs->builder, "cmdHeadersAdd"));
 	gtk_widget_set_sensitive ((GtkWidget *) prefs->add_header, FALSE);
